declare -a OPENSOURCE_APP_LIST
declare -a IFX_APP_LIST
declare -a FEATURE_LIBS
declare -a COM_LIBS
declare -a TARGETS
ROOTFS_SIZE=0;
RC=0
################# COMMON FUNCTIONS #################

build_help()
{
	echo "Usage: $0 [clean configure]"
	echo "clean: clean binary and object files"
	echo "configure: use make configure to configure the complie options."
}
parse_args()
{
	BUILD_CLEAN=0
	BUILD_CONFIGURE=0
	while [ $# -ge 1 ]; do
		case $1 in
		distclean | clean) 
			BUILD_CLEAN=1
			clean="$1";;
		configure) 
			BUILD_CONFIGURE=1;;
		config_only)
		    ;;
		*) ;;
		esac
		shift
	done
}
display_info ()
{
	echo -en "\033[1;34m"
	echo -e "$1"
	echo -en "\033[0m"
}
display_success()
{
	echo -en "\033[1;32m"
	echo -e "$1"
	echo -en "\033[0m"
}
display_error()
{
	echo -en "\033[1;31m"
	echo -e "ERROR : $1"
	echo -en "\033[0m"
	exit 1
}
add_menu()
{
	comment="$1"
	echo "mainmenu_option next_comment" >> \
		"$BUILD_TOOLS_DIR"/appsconfig
	echo "comment '$comment'" >> \
		"$BUILD_TOOLS_DIR"/appsconfig
}
add_option()
{
	dir="$1"
	option="`echo $dir | tr '[:punct:]' '_'`"
	CONFIG="`echo $option | tr '[:lower:]' '[:upper:]'`"
	echo "bool '`echo $option`' BUILD_`echo $CONFIG`" >> \
		"$BUILD_TOOLS_DIR"/appsconfig
}
endmenu()
{
	MENU="$1"
	echo "endmenu" >> \
		"$BUILD_TOOLS_DIR"/appsconfig

}
setup_link ()
{
	#if [ -e "$1" ]; then
		ln -sf "$1" "$2"
	#else
		#display_error "\n$1 does not exist.\nLink Setup failed"
	#fi
}
has_model_changed()
{
	if [ -e .config.old ];then
		ORIG=`grep 'MODEL=' .config.old | sed "s,MODEL=,,"`
	fi
	if [ -e .config ];then
		NEW=`grep 'MODEL=' .config | sed "s,MODEL=,,"`
	fi
	if [ "$NEW" != "$ORIG" ];then
		return 1
	else
		return 0
	fi
}
ifx_error_check()
{
        if [ $1 -ne 0 ]; then
                display_error "Build failed"
        fi
}

set_tc_path()
{
	if [ "$CONFIG_TC_BOTH" = "y" ]; then
		export PATH=`echo "$PATH" | sed "s,gcc-3.4.4,gcc-3.3.6,"`
		export CCACHE_PATH=`echo "$CCACHE_PATH" | sed "s,gcc-3.4.4,gcc-3.3.6,"`
	fi
}

restore_tc_path()
{
	if [ "$CONFIG_TC_BOTH" = "y" ]; then
		export PATH=`echo "$PATH" | sed "s,gcc-3.3.6,gcc-3.4.4,"`
		export CCACHE_PATH=`echo "$CCACHE_PATH" | sed "s,gcc-3.3.6,gcc-3.4.4,"`
	fi

}
################# LIBRARY FUNCTIONS #################

build_uclibc()
{
	UCLIBC_CONFIG="$1"
	if [ "$UCLIBC_CONFIG" != "" ]; then
	    display_info "Building uclibc with "$UCLIBC_CONFIG" configuration"
	else
	    exit 0;
	fi
	cd "$TOOLCHAIN_DIR"
	rm -rf uClibc*
	if [ ! -e "$KERNEL_SOURCE_DIR"/include/linux/autoconf.h ]; then
		touch "$KERNEL_SOURCE_DIR"/include/linux/autoconf.h
	fi	
	cp source/buildroot/dl/uClibc-0.9.27.tar.bz2 .
	tar xjf uClibc-0.9.27.tar.bz2
	cd uClibc-0.9.27
	case "$UCLIBC_CONFIG" in
		optimized)
			cp "$UCLIBC_CONFIG"/ifx-uClibc-opt-config."$CONFIG_ENDIAN" ./.config
			sed -i "s,^KERNEL_SOURCE.*,KERNEL_SOURCE=\"$KERNEL_SOURCE_DIR\"," .config
			make oldconfig;;
		custom)
			display_info "Now please configure uClibc ..."
			sleep 3
			cp "$UCLIBC_CONFIG"/ifx-uClibc-def-config."$CONFIG_ENDIAN" ./.config
			sed -i "s,^KERNEL_SOURCE.*,KERNEL_SOURCE=\"$KERNEL_SOURCE_DIR\"," .config
			make menuconfig;;
		default)
			cp "$UCLIBC_CONFIG"/ifx-uClibc-def-config."$CONFIG_ENDIAN" ./.config
			sed -i "s,^KERNEL_SOURCE.*,KERNEL_SOURCE=\"$KERNEL_SOURCE_DIR\"," .config
			make oldconfig;;
	        *);;
	esac
	make CROSS=""$CONFIG_ENDIAN"-linux-" TARGET_ARCH=mips KERNEL_SOURCE="$KERNEL_SOURCE_DIR" 
	make install PREFIX="$TOOLCHAIN_DIR"/toolchain-"$CONFIG_ENDIAN"/
	#if [ "$BUILD_ROOTFS_DIR" = "y" ]; then
		#make install_runtime PREFIX="$BUILD_ROOTFS_DIR"
	#fi
	cd "$TOOLCHAIN_DIR"
	rm -rf uClibc*
	cd "$TOOLCHAIN_SOURCE_PATH"
}

select_other_libs()
{
        if [ "$CONFIG_TC_3_3_6" = "y" ]; then
		if [ "$CONFIG_CPP_LIB" = "y" ]; then
			FEATURE_LIBS="$FEATURE_LIBS libstdc++.so.5.0.7"
		fi
		if [ "$CONFIG_NCURSES_LIB" = "y" ]; then
			FEATURE_LIBS="$FEATURE_LIBS libncurses.so.5.2 libmenu.so.5.2 libpanel.so.5.2 libform.so.5.2 libcurses.so"
		fi
		if [ "$CONFIG_Z_LIB" = "y" ]; then
			FEATURE_LIBS="$FEATURE_LIBS libz.so.1.2.2"
		fi
	else
		if [ "$CONFIG_CPP_LIB" = "y" ]; then
			FEATURE_LIBS="$FEATURE_LIBS libstdc++.so.6.0.3"
		fi
		if [ "$CONFIG_NCURSES_LIB" = "y" ]; then
			FEATURE_LIBS="$FEATURE_LIBS libncurses.so.5.5 libmenu.so.5.5 libpanel.so.5.5 libform.so.5.5 libcurses.so"
		fi
		if [ "$CONFIG_Z_LIB" = "y" ]; then
			FEATURE_LIBS="$FEATURE_LIBS libz.so.1.2.3"
		fi
	fi
        if [ "$CONFIG_OPENSSL_LIB" = "y" ]; then
                FEATURE_LIBS="$FEATURE_LIBS libssl.so.0.9.7 libcrypto.so.0.9.7"
        fi
        if [ "$CONFIG_CRYPTO" = "y" ]; then
                FEATURE_LIBS="$FEATURE_LIBS libcrypto.so.0.9.7"
        fi
        if [ "$CONFIG_OPENSSL_SSL" = "y" ]; then
                FEATURE_LIBS="$FEATURE_LIBS libssl.so.0.9.7"
        fi

	select_lib # selects any project specific libraries
}
check_lib()
{
	for file in `cat "$TOOLS_DIR"/.libs`
	do
		if [ "$1" = "$file" ]; then
			RC=1
			return
		fi
	done
	RC=0
}

get_libs()
{
	lib_found=1;
	for lib in $@
	do
		echo "$lib" >> "$TOOLS_DIR"/.libs
	done
	while [ $lib_found -eq 1 ];
	do
		lib_found=0
		for link in `find "$LIBPATH" -maxdepth 1 -type l -printf "%f\n"`
		do
			target=`find "$LIBPATH" -maxdepth 1 -name "$link" -printf "%l\n"`
			check_lib "$target"
			if [ $RC -eq 1 ]; then
				check_lib "$link"
				if [ $RC -eq 0 ] ;then
					echo "$link" >> "$TOOLS_DIR"/.libs
				fi
			fi
			
		done
	done

}
cp_libs()
{
	cd "$LIBPATH"
	if [ "$CONFIG_TC_3_3_6" != "y" ]; then
		COM_LIBS=( "libgcc_s.so.1" )
	fi
	for lib in `ls *0.9.28*`
	do 
		COM_LIBS=""$COM_LIBS" "$lib""
	done
	if [ "$CONFIG_LIB_FEATURE" = "y" ]; then
		select_other_libs
	fi

	# Pramod - include ssl libraries even if one of SNMPv3 or TR69 is defined
	if [ "$IFX_CONFIG_SNMPv3" = "1" -o "$IFX_CONFIG_TR69" = "1" ]; then
		if [ "$CONFIG_MATRIX" != "1" ];then
				FEATURE_LIBS="$FEATURE_LIBS libssl.so.0.9.7 libcrypto.so.0.9.7"
		fi
	fi

	# leejack - include related libraries of thomson wireless module	
	if [ "$IFX_CONFIG_WIRELESS_TSC" = "1" ]; then
		FEATURE_LIBS="$FEATURE_LIBS libargtable2.so.0.1.1 libpcap.a"
		FEATURE_LIBS="$FEATURE_LIBS libpcap.so.0.9.5 libcrypto.so.0.9.7"
	fi

	TARGETS=""$COM_LIBS" "$FEATURE_LIBS""
	get_libs ${TARGETS[@]}
	install -d "$BUILD_ROOTFS_DIR"/lib
	for lib in `cat "$TOOLS_DIR"/.libs` 
	do
		cp -af "$LIBPATH"/"$lib" "$BUILD_ROOTFS_DIR"/lib
	done
	cd "$TOOLS_DIR"
	mv .libs .libs.old
}
rm_libs()
{
	if [ -e "$TOOLS_DIR"/.libs.old ]; then
		for lib in `cat "$TOOLS_DIR"/.libs.old` 
		do
			rm -f "$BUILD_ROOTFS_DIR"/lib/"$lib"
		done
	fi
}

################# ROOTFS BUILD FUNCTIONS #################

create_rootfs()
{
	parse_args $@
	rm -f "$PLATFORM_DIR"/source/rootfs/flashdisk/etc/ppp/options > /dev/null
	rm -f "$PLATFORM_DIR"/source/rootfs/flashdisk/etc/ppp/resolv.conf > /dev/null
	rm -f "$PLATFORM_DIR"/source/rootfs/flashdisk/etc/ppp/peers > /dev/null
	if [ ! -d "$BUILD_ROOTFS_DIR" ]; then
		mkdir -p "$BUILD_ROOTFS_DIR"
		if [ $? -ne 0 ]; then
			display_error "Failed to create build directories"
		fi
		cp -ar "$ROOTFS_SOURCE_DIR"/* "$BUILD_ROOTFS_DIR"
	fi
	if [ "$COPY_TFTP" = "y" -a ! -d "$TFTP_DIR" ];then
		mkdir -p "$TFTP_DIR"
	fi
	rm_libs
	cp_libs
	if [ "$BUILD_CONFIGURE" -eq 1 ];then
		rm -rf "$BUILD_ROOTFS_DIR"/etc/rc.d
		mkdir -p "$BUILD_ROOTFS_DIR"/etc
		cp -a "$ROOTFS_SOURCE_DIR"/etc/rc.d "$BUILD_ROOTFS_DIR"/etc/.
	fi
	if [ "$IFX_ADSL_FIRMWARE_IN_ROOTFS" = "1" ]; then
		build_firmware
	fi

}

build_ext2_rootfs ()
{
	grep -q "CONFIG_EXT2_FS=y" "$KERNEL_CONFIG"
	if [ $? -eq 0 ]; then
		GENEXT2_ADDTOINODESIZE=`find "$BUILD_ROOTFS_DIR" | wc -l`
		GENEXT2_INODES=`expr $GENEXT2_ADDTOINODESIZE + 50`
		fakeroot -i "$FAKE_ENV" -s "$FAKE_ENV" \
		-- genext2fs -b $ROOTFS_SIZE -i $GENEXT2_INODES \
		-d "$BUILD_ROOTFS_DIR" "$BUILD_DIR"/rootfs
		if [ $? -ne 0 ]; then
			display_error "Building ext2 rootfs failed."
		fi
	else
		display_error "No EXT2 Support in the kernel. Cannot build EXT2 rootfs"
	fi
}

build_jffs2_rootfs ()
{
        grep -q "CONFIG_JFFS2_FS=y" "$KERNEL_CONFIG"
        if [ $? -eq 0 ]; then
                display_info "\nCreating JFFS2 image for small page NAND flash\n"
                fakeroot -i "$FAKE_ENV" -s "$FAKE_ENV" \
                -- mkfs.jffs2 -b -v -n -e 0x4000 -p 0x4000 -d "$BUILD_ROOTFS_DIR" \
                -o "$BUILD_DIR"/rootfs
                if [ $? -ne 0 ]; then
                        exit 1;
                fi
        else
                display_error "No JFFS2 Support in the kernel. Cannot build JFFS2 rootfs"
        fi
}

build_sqsh_rootfs ()
{
	grep -q "CONFIG_SQUASHFS=y" "$KERNEL_CONFIG"
	SQUASHFS=$?
	if [ "$SQUASHFS" -eq 0 ]; then
		grep -q "CONFIG_SQUASHFS_LZMA=y" "$KERNEL_CONFIG"
		LZMA=$?
		if [ "$CONFIG_MIPS" = "y" ]; then
			FS_OPT="-be"
		else
			FS_OPT="-le"
		fi
		if [ $LZMA -eq 0 ]; then
			display_info "\nCreating Squashfs-lzma image\n"
			TOOL="mksquashfs-lzma"
		else
			display_info "\nCreating Squashfs image\n"
			TOOL="mksquashfs"
		fi
		fakeroot -i "$FAKE_ENV" -s "$FAKE_ENV" -- \
		"$TOOL" "$BUILD_ROOTFS_DIR" "$BUILD_DIR"/rootfs "$FS_OPT" -noappend
		fakeroot -i "$FAKE_ENV" -s "$FAKE_ENV" -- \
		mkimage -A MIPS -O Linux -C lzma -T standalone -e 0x00 \
		display_error "\n Finish Build Squarshfs image ..... !!!\n"
		-a 0x00 -n "$MODEL RootFS" -d "$BUILD_DIR"/rootfs "$BUILD_DIR"/rootfs.img
		rm -f "$BUILD_DIR"/rootfs
	else 
		display_error "No SQUASHFS Support in the kernel. Cannot build SQUASHFS rootfs"
	fi
}

build_ramdisk()
{
	mv "$BUILD_DIR"/rootfs "$BUILD_DIR"/ramdisk
	if [ "$CONFIG_EMBEDDED_RD" = "y" ]; then
    		cp -f "$BUILD_DIR"/ramdisk .
    		gzip -9 ramdisk
    		rm -f "$KERNEL_SOURCE_DIR"/arch/mips/ramdisk/ramdisk.*
    		cp -f ramdisk.gz "$KERNEL_SOURCE_DIR"/arch/mips/ramdisk
    		rm -f ramdisk ramdisk.gz 2> /dev/null
		sed -i "s,^CONFIG_BLK_DEV_RAM_SIZE.*,CONFIG_BLK_DEV_RAM_SIZE=$ROOTFS_SIZE," "$KERNEL_CONFIG"
	fi
}

write_to_file()
{
	FILE="$1"
	shift
	for obj in $@
	do
		grep -qx "$obj" "$FILE" &> /dev/null
		if [ $? -ne 0 ];then
			echo "$obj" >> "$FILE"
		fi
	done
}


prepare_rootfs()
{
	rm -f "$FAKE_ENV" 2> /dev/null
	touch "$FAKE_ENV"
	#find "$BUILD_ROOTFS_DIR" -type f -perm +111 | \
	fakeroot -i "$FAKE_ENV" -s "$FAKE_ENV" -- \
	find "$BUILD_ROOTFS_DIR" | xargs "$IFX_STRIP" -R.note -R.comment &>/dev/null
	/sbin/ldconfig -r $"BUILD_ROOTFS_DIR" 2>/dev/null
	fakeroot -i "$FAKE_ENV" -s "$FAKE_ENV" -- \
	chown -R root:root "$BUILD_ROOTFS_DIR"
	fakeroot -i "$FAKE_ENV" -s "$FAKE_ENV" -- \
	makedevs -d "$DEV_FILE" "$BUILD_ROOTFS_DIR"
	ROOTFS_SIZE=`LANG=C du -l -s -c -k "$BUILD_ROOTFS_DIR"\
	| grep total | sed -e "s/total//"`
        cd "$BUILD_DIR"
        #fakeroot -i "$FAKE_ENV" -s "$FAKE_ENV" -- \
        #tar -czf rootfs.tar.gz root_filesystem
        cd "$BUILD_TOOLS_DIR"
}
build_rootfs()
{
	display_info "\nBuilding Rootfs\n"
	if [ "$CONFIG_FS_NONE" != "y" ]; then
		create_rootfs $@
		if [ "$CONFIG_FS_OTHER" = "y" ]; then
			cd "$BUILD_DIR"
			rm_libs
			cp_libs
			cd "$BUILD_DIR"
			mv -f libuClibc-0.9.28.so libcrypto.so.0.9.7 \
			libpthread-0.9.28.so libm-0.9.28.so libssl.so.0.9.7 \
			"$BUILD_ROOTFS_DIR"/lib &>/dev/null
			find . -maxdepth 1 -name '*.so' | xargs rm -f
			rm -f unresolved needed '*.a'
			cd "$TOOLS_DIR"
			build_cust_rootfs
		else
			prepare_rootfs
		fi
		if [ "$CONFIG_FS_EXT2" = "y" ]; then
			build_ext2_rootfs
		fi
		if [ "$CONFIG_FS_JFFS2" = "y" ]; then
			build_jffs2_rootfs
		fi
		if [ "$CONFIG_FS_SQUASHFS" = "y" ]; then
			build_sqsh_rootfs
		fi
		if [ "$CONFIG_RAMD" = "y" ]; then
			build_ramdisk
		fi
		#if [ "$COPY_TFTP" = "y" -a -e "$BUILD_DIR"/rootfs ]; then
			#cp -f "$BUILD_DIR"/rootfs "$TFTP_DIR"
		#fi
		rm -f "$FAKE_ENV" 2>/dev/null
	fi
}

################# UBOOT BUILD FUNCTIONS #################
build_uboot()
{
	display_info "\nBuilding U-Boot\n"
	cd "$UBOOT_SOURCE_DIR"
	echo $@
	./build.sh $@
	ifx_error_check $?
	cd "$BUILD_TOOLS_DIR"
}

################# KERNEL BUILD FUNCTIONS #################

build_kernel_modules()
{
	#build kernel modules
	cd "$KERNEL_SOURCE_DIR"
	make modules
	IFX_BUILD_ERRNO=$?
	if [ $IFX_BUILD_ERRNO -ne 0 ]
	then
		display_error "Kernel module Fail!"
	fi

	make modules_install INSTALL_MOD_PATH="$BUILD_ROOTFS_DIR"
	IFX_BUILD_ERRNO=$?
	if [ $IFX_BUILD_ERRNO -ne 0 ]
	then
		display_error "Installing Kernel module Fail!"
	fi

	cd -
	find ${BUILD_ROOTFS_DIR}lib/modules -name *.o |xargs mips-linux-strip --strip-unneeded
	rm -rf ${BUILD_ROOTFS_DIR}lib/modules/2.4.31-`cat $KERNEL_SOURCE_DIR/.hhl_target_lspname`/pcmcia
	rm -f ${BUILD_ROOTFS_DIR}lib/modules/2.4.31-`cat $KERNEL_SOURCE_DIR/.hhl_target_lspname`/modules.dep
	${BUILD_TOOLS_DIR}depmod.pl -b ${BUILD_ROOTFS_DIR}lib/modules/2.4.31-`cat $KERNEL_SOURCE_DIR/.hhl_target_lspname` -k ${KERNEL_SOURCE_DIR}/vmlinux -F ${KERNEL_SOURCE_DIR}/System.map
}

change_endian_if_needed()
{
	grep -q 'CONFIG_CPU_LITTLE_ENDIAN=y' "$KERNEL_CONFIG"
	LITTLE=$?
	if [ "$CONFIG_MIPSEL" = "y" ]; then
		if [ $LITTLE -ne 0 ]; then
			sed -i "s/# CONFIG_CPU_LITTLE_ENDIAN.*/CONFIG_CPU_LITTLE_ENDIAN=y/" "$KERNEL_CONFIG"
		fi
	else
		if [ $LITTLE -eq 0 ]; then
			sed -i "s/CONFIG_CPU_LITTLE_ENDIAN=y/# CONFIG_CPU_LITTLE_ENDIAN is not set/" "$KERNEL_CONFIG"
		fi
	fi
}

config_ramd()
{
	grep -q 'CONFIG_BLK_DEV_RAM=y' "$KERNEL_CONFIG"
	RAMD=$?
	grep -q 'CONFIG_BLK_DEV_INITRD=y' "$KERNEL_CONFIG"
	INITRD=$?
	grep -q 'CONFIG_EMBEDDED_RAMDISK=y' "$KERNEL_CONFIG"
	EMBEDDED=$?
	if [ "$CONFIG_RAMD" = "y" ]; then
		if [ $RAMD -ne 0 ]; then
			sed -i "s,# CONFIG_BLK_DEV_RAM.*,CONFIG_BLK_DEV_RAM=y,"\
			"$KERNEL_CONFIG"
			sed -i "/CONFIG_BLK_DEV_RAM/a \CONFIG_BLK_DEV_RAM_SIZE=4096"\
			"$KERNEL_CONFIG"
		fi
		if [ "$CONFIG_EMBEDDED_RD" = "y" ]; then
			if [ $INITRD -ne 0 ]; then
				sed -i "s,# CONFIG_BLK_DEV_INITRD.*,CONFIG_BLK_DEV_INITRD=y," "$KERNEL_CONFIG"
			fi
			if [ $EMBEDDED -ne 0 ]; then
				sed -i "s,# CONFIG_EMBEDDED_RAMDISK.*,CONFIG_EMBEDDED_RAMDISK=y," "$KERNEL_CONFIG"
				sed -i "/CONFIG_EMBEDDED_RAMDISK/a \CONFIG_EMBEDDED_RAMDISK_IMAGE=\"ramdisk.gz\"" "$KERNEL_CONFIG"
	
			else
				sed -i "/CONFIG_EMBEDDED_RAMDISK_IMAGE/c \CONFIG_EMBEDDED_RAMDISK_IMAGE=\"ramdisk.gz\"" "$KERNEL_CONFIG"
			fi
		else
			if [ $EMBEDDED -eq 0 ]; then
				sed -i "s,CONFIG_EMBEDDED_RAMDISK=y,# CONFIG_EMBEDDED_RAMDISK is not set," "$KERNEL_CONFIG"
				sed -i "s,CONFIG_EMBEDDED_RAMDISK_IMAGE.*,," "$KERNEL_CONFIG"
			fi
		fi
	else
		if [ $RAMD -eq 0 ]; then
			sed -i "s,CONFIG_BLK_DEV_RAM.*,# CONFIG_BLK_DEV_RAM is not set," "$KERNEL_CONFIG"
		fi
	fi
}

clean_configure_kernel()
{
	cd "$KERNEL_SOURCE_DIR"
	parse_args $@
	if [ $BUILD_CLEAN -eq 1 ]; then
		make "$clean"
		rm -f "$BUILD_DIR"/uImage 2> /dev/null
		rm -f "$BUILD_DIR"/vmlinux.srec 2> /dev/null
	fi
	if [ $BUILD_CONFIGURE -eq 1 ]; then
		if [ arch/mips/defconfig -nt .config ]; then
			display_info "\nUsing default config\n"
			sleep 1
			cp -Lf arch/mips/defconfig .config 
		fi
		chmod a+w .config
		change_endian_if_needed
		config_ramd
		make oldconfig
		make dep
	fi
	cd - 2>/dev/null
}

compile_kernel()
{
    	cd "$KERNEL_SOURCE_DIR"
	if [ "$CONFIG_RAMD" = "y" ]; then
		make oldconfig; #To update ramdisk size
	fi
        if [ "$CONFIG_UIMAGE_GZIP" = "y" ]; then
                make uImage.gzip
        elif [ "$CONFIG_UIMAGE_BZIP2" = "y" ]; then
                make uImage.bzip2
        elif [ "$CONFIG_UIMAGE_LZMA" = "y" ]; then
                make IFX_SMALL_FOOTPRINT=1
                #make IFX_SMALL_FOOTPRINT=1 uImage.lzma
                make IFX_SMALL_FOOTPRINT=1 uImage
                #make uImage.lzma
        else
                make vmlinux.srec
        fi

}
build_kernel()
{
	set_tc_path
	for arg in $@ 
	do
		display_info "\nBuilding Kernel - $arg\n"
		case "$arg" in
		clean | distclean | configure)
			clean_configure_kernel $arg;;
		modules)
			build_kernel_modules;;
		image)
			compile_kernel;;
		*);;
		esac
	done
	restore_tc_path
}

################# APP BUILD FUNCTIONS #################
get_app_ver()
{
	if [ -e version.h ]; then
		name=`grep "#define APP_NAME" version.h | sed "s/#define APP_NAME//"`
		ver=`grep "#define APP_VER" version.h | sed "s/#define APP_VER//"`
		echo -n $name >> "$VER_FILE"
		echo -ne "\t" >> "$VER_FILE"
		echo $ver >> "$VER_FILE"
	fi
}
setup_one_app()
{
	dir="$1"
	if [ -e "$dir" ];then
		cd "$dir"
		get_app_ver
		rm -f sysconfig.sh config.sh model_config.sh
		setup_link "$BUILD_TOOLS_DIR"/.config sysconfig.sh
		setup_link "$BUILD_TOOLS_DIR"/featureconfig.sh config.sh
		setup_link "$BUILD_TOOLS_DIR"/modelconfig.sh model_config.sh
		add_option "`basename $dir`"
		cd - > /dev/null
	else
		display_error "$dir does not exist"
	fi

}

apps_setup()
{
	cd "$BUILD_TOOLS_DIR"
    	rm -f "$VER_FILE"
	rm -f appsconfig
	select_ifx_apps
	add_menu "IFX applications" appsconfig.menu
	for dir in ${IFX_APP_LIST[$@]}
	do
		setup_one_app "$USER_IFXSOURCE_DIR"/"$dir"
	done
	endmenu appsconfig.menu
    	select_opensource_apps
	add_menu "Opensource applications" appsconfig.menu
	for dir in ${OPENSOURCE_APP_LIST[$@]}
	do
		setup_one_app "$USER_OPENSOURCE_DIR"/"$dir"
	done
	endmenu appsconfig.menu
	if [ -e featureconfig.sh ]; then
		cp -f "$BUILD_TOOLS_DIR"/featureconfig.sh "$BUILD_ROOTFS_DIR"/etc/config.sh 2> /dev/null
		cp -f "$BUILD_TOOLS_DIR"/featureconfig.sh "$ROOTFS_SOURCE_DIR"/etc/config.sh 2> /dev/null
	fi
	if [ -e "${VER_FILE}" ]; then
		cp -f "$VER_FILE" "$BUILD_ROOTFS_DIR"/etc 2> /dev/null
		mv -f "$VER_FILE" "$ROOTFS_SOURCE_DIR"/etc 2> /dev/null
	fi
	cd "$BUILD_TOOLS_DIR"
}

build_tapi()
{
    cd ${USER_IFXSOURCE_DIR}tapi
    ./build.sh $@
    IFX_BUILD_ERRNO=$?
    if [ $IFX_BUILD_ERRNO -ne 0 ]
    then
        echo "Build TAPI drivers fail!"
        exit $IFX_BUILD_ERRNO
    fi

}

build_opensource_apps()
{
        display_info "\nBuilding opensource apps\n"
	select_opensource_apps
        for dir in ${OPENSOURCE_APP_LIST[@]}
        do
		cd "$USER_OPENSOURCE_DIR"/"$dir"
		./build.sh $@
		IFX_BUILD_ERRNO=$?
		if [ $IFX_BUILD_ERRNO -ne 0 ]
		then
			display_error "Build $dir fail!"
		fi
        done
	cd "$BUILD_TOOLS_DIR"
}

build_ifx_apps()
{
        display_info "\nBuilding ifx apps\n"
	cd "$USER_OPENSOURCE_DIR"/mtd
	./build.sh $@
	cd -
	select_ifx_apps
        for dir in ${IFX_APP_LIST[@]}
        do
			cd "$USER_IFXSOURCE_DIR"/"$dir"
		pwd
			./build.sh $@
			IFX_BUILD_ERRNO=$?
			if [ $IFX_BUILD_ERRNO -ne 0 ]
			then
				display_error "Build $dir fail!"
			fi
        done
	cd "$BUILD_TOOLS_DIR"
}

if [ x"`echo -n`" = "x-n" ] ; then
    prompt_n_nl() {
        echo "$*\c" > /dev/tty
    }
else
    prompt_n_nl() {
        echo -n "$*" > /dev/tty
    }
fi

cd_no_echo() {
    cd "$*" 1>/dev/null
}

prompt_nl() {
    echo "$*"
}

prompt_string() {
    message="$1"
    default="$2"
    if [ "$SP_AUTO" = "1" ]; then
        prompt_nl $default
    else
        ok="no"
        while [ $ok != "yes" ] ; do
            if [ "$default" != "" ] ; then
                prompt_n_nl "$message [$default] "
                read < /dev/tty answer
                if [ "$answer" = "" ] ; then
                    answer="$default"
                    ok="yes"
                    prompt_nl $answer
                    continue
                fi
            else
                prompt_n_nl "$message "
                read < /dev/tty answer
                if [ "$answer" = "" ] ; then
                    ok="no"
                    continue
                fi
            fi
            prompt_nl $answer
            ok="yes"
        done
    fi
}

absolute_path() {
    D=`dirname "$1"`
    N=`basename "$1"`
    if [ "$N" != ".." ] ; then
        cd_no_echo "$D"
        if [ "$N" != "." ]; then
            if [ "`pwd`" = "/" ]; then
                prompt_nl "/$N/"
            else
                prompt_nl "`pwd`/$N/"
            fi
        else
            prompt_nl "`pwd`/"
        fi
        cd_no_echo -
    else
        cd_no_echo "$D"
        absolute_path "$D"
        cd_no_echo -
    fi
}


export PATH="$HOSTUTILS_PATH":"$TOOLCHAIN_PATH":$PATH
export CCACHE_PATH="$CCACHE_PATH"
